#include <LiquidCrystal.h> ; // LCD 라이브러리를 사용하기 위한 헤더 파일을 추가 
#include <Wire.h>
#include "WEMOS_SHT3X.h"

// LCD 설정
//     LCD (RS, Enable, Data Bit 4, 5, 6, 7 )
LiquidCrystal lcd(3, 4, 9, 10, 11, 12);  // LCD와 보드의 핀 번호를 적어 줍니다.
// 온습도 센서 설정
//     I2C Address : 0x45
SHT3X sht30(0x45);


// LCD 표시
//    pstrF : 첫줄 문자열
//    pstrS : 두번째 줄 문자열
void ShowLCD(char* pstrF, char* pstrS)
{
  lcd.setCursor(0, 1);  // LCD에 Display할 좌표를 정해 줍니다.
  lcd.begin(16, 2);    // LCD 데이터를 2행 16열로 표시합니다.
  lcd.clear();   // LCD 화면을 지웁니다.
  lcd.print(pstrF);  // LCD 화면에 “ Hello Blueinno!”를 표시합니다.
  lcd.setCursor(0, 2);  // LCD에 Display할 좌표를 정해 줍니다.
  lcd.print(pstrS);  // LCD 화면에 “ Hello Blueinno!”를 표시합니다.
}

// 이슬점 계산을 위한 테이블
float dewPoint[] =
  // -5     0      5    10   15    20    25    30    35   40
{
  -19.0, -14.3, -10.2, -6.9, -2.9, 1.3,  5.2,  9.2, 13.7, 18.0,  // 30
  -17.5, -12.1, -8.6, -4.7, -0.8, 3.4,  7.4, 12.0, 16.1, 20.6,  // 35
  -15.9, -10.6, -7.3, -3.1, 0.9,  5.4,  9.5, 14.0, 18.2, 23.0,  // 40
  -14.3, -9.6, -5.7, -1.5, 2.6,  7.0, 11.7, 16.0, 20.2, 25.2,  // 45
  -12.8, -8.4, -4.4, -0.3, 4.1,  8.6, 13.3, 17.5, 22.2, 27.1,  // 50
  -11.6, -7.4, -3.2, 1.0, 5.6, 10.4, 14.8, 19.1, 23.9, 28.9,  // 55
  -10.8, -6.5, -2.1, 2.3, 6.7, 11.9, 16.2, 20.6, 25.3, 30.5,  // 60 %
  -10.0, -5.4, -1.0, 3.3, 8.0, 13.0, 17.4, 22.0, 26.8, 32.0,  // 65 %
  -9.2, -4.5, 0.2, 4.5, 9.1, 14.2, 18.6, 23.3, 28.1, 33.5,  // 70 %
  -8.4, -3.6, 0.9, 5.6, 10.4, 15.4, 19.9, 24.7, 29.6, 34.5,  // 75 %
  -7.7, -2.8, 1.9, 6.5, 11.6, 16.5, 21.0, 25.9, 31.0, 36.2,  // 80 %
  -7.2, -2.0, 2.3, 7.3, 12.5, 17.4, 22.1, 27.0, 32.0, 37.1,  // 85 %
  -6.5, -1.3, 3.5, 8.2, 13.3, 18.3, 23.2, 29.0, 33.0, 38.2,  // 90 %
};

float fTempAvg = 0.0f;
float fTempMax = 0.0f;
float fTempMin = 0.0f;
float fHuniAvg = 0.0f;
float fHuniMax = 0.0f;
float fHuniMin = 0.0f;


int iCCC = 0;
int dTemp = 1;
int disType = 0;

// 이슬점 계산 알고리즘
//   fTemp : 섭씨 온도
//   fHumi : 상대 습도
float computeDew(float fTemp, float fHumi)
{
  int iTemp, iHumi;
  int iIndex;
  float dewDegree1, dewDegree2, dewDegree;

  if (fHumi < 30)
    return -100;

  if (fHumi > 90)
    return -100;

  if (fTemp < -5)
    return -100;

  if (fTemp > 40)
    return -100;

  iTemp = ((int)fTemp + 5) / 5;
  iHumi = ((int)fHumi - 30) / 5;

  iIndex = iHumi * 10 + iTemp;

  // 1. First Appriximation
  dewDegree1 = (dewPoint[iIndex + 1] - dewPoint[iIndex]) * (fTemp - iTemp * 5.0 + 5) / 5.0 + dewPoint[iIndex];

  // 2. Second Approximation
  dewDegree2 = (dewPoint[iIndex + 11] - dewPoint[iIndex + 10]) * (fTemp - iTemp * 5.0 + 5) / 5.0 + dewPoint[iIndex + 10];

  // 3. Final
  dewDegree = (dewDegree2 - dewDegree1) * (fHumi - iHumi * 5.0 - 30) / 5.0 + dewDegree1;

  //Serial.println(dewDegree);

  return dewDegree;
}



// Float 변수를 문자열로 변환하는 함수
void F2S(float f, int fSize, int pSize, char* pStr)
{
  int i;
  float a;

  for (i = 0; i < fSize; i++)
  {
    a = f / pow(10, fSize - i - 1);
    pStr[i] = 48 + (int)a % 10;
  }
  pStr[fSize] = '.';

  for (i = 0; i < pSize; i++)
  {
    a = f * pow(10, i + 1);
    pStr[fSize + 1 + i] = 48 + (int)a % 10;
  }
  pStr[fSize + pSize + 1] = 0;
}

void setup()
{
  // 시리얼 포트 설정
  Serial.begin(9600);
  // I2C 포트 설정
  Wire.beginOnPins(2, 1) ; //GPIO 핀에 사용자 할당
  // LCD 설정
  lcd.begin(16, 2);    // LCD 데이터를 2행 16열로 표시합니다.
}




void loop() {

  sht30.get();

  char strFirst[17];
  char strSecond[17];
  int i1, i2, i3,i4;
  int iTempC;
  int bPoint;
  float dew;
  float Temp;
  float Humi;
  float Humidex;
  char str1[10];
  char str2[10];
  char str3[10];
  char str4[10];

  Temp = sht30.cTemp;
  Humi = sht30.humidity;
  dew = computeDew(sht30.cTemp, sht30.humidity);
  Humidex = 1.8 * Temp - 0.55 * (1 - Humi / 100) * (1.8 * Temp - 26) + 32;
  F2S(sht30.cTemp, 2, 2, str1);
  F2S(sht30.humidity, 2, 2, str2);
  F2S(Humidex, 2, 2, str4);
  if (dew != -100)
    F2S(dew, 2, 2, str3);
  else
    sprintf(str3, "Empty");

  sprintf(strFirst, "T %s H %s", str1, str2);
  sprintf(strSecond, "D %s I %s", str3, str4);


  ShowLCD(strFirst, strSecond);


  // Output data to the serial monitor


  Serial.println(Humidex);
  Serial.println(sht30.cTemp);
  Serial.println(sht30.humidity);

  iCCC = iCCC + 1;

  delay(5);

}
